import logging
import asyncio
import csv
import json
from argparse import ArgumentParser
from pathlib import Path
from typing import Dict

from .functions import get_all_financials, ResultDict
from .constants import END_DATE, MAIN_LOGGER_NAME, DEFAULT_CONCURRENT_BATCH_SIZE


log = logging.getLogger(MAIN_LOGGER_NAME)

JSON_EXT, CSV_EXT = '.json', '.csv'

TICKER_SYMBOL = 'ticker_symbol'
QUARTERLY = 'quarterly'
BATCH_SIZE = 'concurrent_batch_size'
TO_FILE = 'to_file'
JSON_INDENT = 'json_indent'
VERBOSE = 'verbose'


def parse_cli() -> dict:
    parser = ArgumentParser(description="Scrape company financials")
    parser.add_argument(
        TICKER_SYMBOL,
        type=str,
        nargs='+',
        help="Stock ticker symbol of the company to be scraped the financials of"
    )
    parser.add_argument(
        '-Q', f'--{QUARTERLY}',
        action='store_true',
        help="If set, the financial data for the last quarters is returned; otherwise yearly data is returned."
    )
    parser.add_argument(
        '-b', f'--{BATCH_SIZE.replace("_", "-")}',
        type=int,
        default=DEFAULT_CONCURRENT_BATCH_SIZE,
        help="If multiple ticker symbols are passed, the company financials can be scraped concurrently. "
             "This argument determines how many datasets are scraped concurrently at any moment in time. "
             "By default, they are scraped sequentially."
    )
    parser.add_argument(
        '-f', f'--{TO_FILE.replace("_", "-")}',
        type=Path,
        help="Writes results to the specified destination file. If omitted results are printed to stdout. "
             "Depending on the file name suffix, the output format can be either as CSV or in JSON."
    )
    parser.add_argument(
        f'--{JSON_INDENT.replace("_", "-")}',
        type=int,
        help="If set to a positive integer and the output format is JSON (default), the resulting JSON document is "
             "indented accordingly for more readability; if omitted, output is returned in one line."
    )
    parser.add_argument(
        '-v', f'--{VERBOSE}',
        action='count',
        default=0,
        help="Verbose mode. Reduces the log level and thus prints out more status messages while running. "
             "Using this flag multiple times increases verbosity further."
    )
    return vars(parser.parse_args())


def configure_logging(verbosity: int) -> None:
    root_logger = logging.getLogger()
    root_logger.addHandler(logging.StreamHandler())
    if verbosity > 2:
        root_logger.setLevel(logging.DEBUG)
    elif verbosity == 2:
        root_logger.setLevel(logging.INFO)
    elif verbosity == 1:
        root_logger.setLevel(logging.WARNING)


def write_to_csv(data: Dict[str, ResultDict], file_obj) -> None:
    writer = csv.writer(file_obj)
    for statement_key, statement_dict in data.items():
        end_dates = statement_dict.pop(END_DATE)
        writer.writerow([statement_key] + list(end_dates))
        for key, values in statement_dict.items():
            writer.writerow([key] + list(str(val) for val in values))


async def main() -> None:
    args = parse_cli()
    configure_logging(args[VERBOSE])
    data = await get_all_financials(*args[TICKER_SYMBOL], quarterly=args[QUARTERLY],
                                    concurrent_batch_size=args[BATCH_SIZE])
    path: Path = args[TO_FILE]
    if path is None:
        print(json.dumps(data, indent=args[JSON_INDENT]))
        return
    with open(path, 'w') as f:
        if path.suffix.lower() == CSV_EXT:
            write_to_csv(data, f)
            return
        if not path.suffix.lower() == JSON_EXT:
            log.warning(f"Extension '{path.suffix}' unknown; using JSON format")
        json.dump(data, f, indent=args[JSON_INDENT])


if __name__ == '__main__':
    asyncio.run(main())
